@@ -43,6 +43,10 @@
  */
 #if JS_HAS_FILE_OBJECT
 
+#ifdef OSSP
+#include "../config.h"
+#endif
+
 #include "jsstddef.h"
 
 /* ----------------- Platform-specific includes and defines ----------------- */
@@ -807,7 +811,11 @@ js_FileOpen(JSContext *cx, JSObject *obj, JSFile *file, char *mode){
 
 /* Buffered version of PR_Read. Used by js_FileRead */
 static int32
+#ifdef OSSP
+js_BufferedRead(JSFile * f, unsigned char *buf, int32 len)
+#else
 js_BufferedRead(JSFile * f, char *buf, int32 len)
+#endif
 {
     int32 count = 0;
 
@@ -901,7 +909,11 @@ js_FileRead(JSContext *cx, JSFile *file, jschar *buf, int32 len, int32 mode)
         break;
 
       case UCS2:
+#ifdef OSSP
+        count = js_BufferedRead(file, (unsigned char*)buf, len*2) >> 1;
+#else
         count = js_BufferedRead(file, (char*)buf, len*2) >> 1;
+#endif
         if (count == -1)
             return 0;
 
@@ -2259,8 +2271,15 @@ static JSBool
 file_toString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval*rval)
 {
     JSFile *file = JS_GetInstancePrivate(cx, obj, &file_class, NULL);
+#ifdef OSSP
+    JSString *str;
 
+    if ((str = JS_NewStringCopyZ(cx, file->path)) == NULL)
+        return JS_FALSE;
+    *rval = STRING_TO_JSVAL(str);
+#else
     *rval = STRING_TO_JSVAL(JS_NewStringCopyZ(cx, file->path));
+#endif
     return JS_TRUE;
 }
 
@@ -2270,6 +2289,10 @@ file_toURL(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
     JSFile *file = JS_GetInstancePrivate(cx, obj, &file_class, NULL);
     char url[MAX_PATH_LENGTH];
     jschar *urlChars;
+#ifdef OSSP
+    size_t len;
+    JSString *str;
+#endif
 
 	JSFILE_CHECK_NATIVE("toURL");
 
@@ -2277,12 +2300,14 @@ file_toURL(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
     /* TODO: js_escape in jsstr.h may go away at some point */
 
 #ifdef OSSP /* BUGFIX */
-{
-    size_t len = strlen(url);
-    urlChars = js_InflateString(cx, url, &len);
-    if (urlChars == NULL) return JS_FALSE;
-    *rval = STRING_TO_JSVAL(js_NewString(cx, urlChars, len, 0));
-}
+    len = strlen(url);
+    if ((urlChars = js_InflateString(cx, url, &len)) == NULL)
+        return JS_FALSE;
+    if ((str = js_NewString(cx, urlChars, len, 0)) == NULL) {
+        JS_free(cx, urlChars);
+        return JS_FALSE;
+    }
+    *rval = STRING_TO_JSVAL(str);
 #else
     urlChars = js_InflateString(cx, url, strlen(url));
     if (urlChars == NULL) return JS_FALSE;
@@ -2675,7 +2700,15 @@ file_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
             goto out;
         }
 
+#if defined(HAVE_STRUCT_STAT_ST_BIRTHTIME)
         t = (time_t)(info.st_birthtime);
+#elif defined(HAVE_STRUCT_STAT_ST_BIRTHTIMENSEC)
+        t = (time_t)(info.st_birthtimensec / (1000*1000*1000));
+#else
+        /* not really correct, but the only possible fallback on not
+           fully POSIX compliant platforms. */
+        t = (time_t)(info.st_ctime);
+#endif
         tm = localtime(&t);
         *vp = OBJECT_TO_JSVAL(js_NewDateObject(cx, tm->tm_year + 1900,
                                     tm->tm_mon + 1,
